/***************************************************************************
                          peer.cpp  -  description
                             -------------------
    begin                : Fri Nov 2 2001
    copyright            : (C) 2001 by Constantinos A. Kotsokalis
    email                : ckotso@netmode.ntua.gr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "client.h"

Client::Client(TCPSocket& lsock, InetHostAddress remaddr, Collector *collector) : TCPSession(lsock){
	// Init properties
	cl = collector;
	prevdetector = remaddr;
	// Detach from parent thread
	Detach();
}

Client::~Client() {

}

void Client::Final() {
	delete this;
}

void Client::Run(){
	string buff;
	char line[520];
	// *tcp() represents the TCP stream from which we read data and to which
	// we write data. It is used like any i/o stream.
	tcp()->getline(line, 520);

	if (!isConnected())
		return;

	buff = string(line);

	// If the data that was sent to us was of zero length (i.e. connection was closed before
	// we manage to read in any data), exit the thread
	if (buff.empty())
		return;

	// ...or else read the data in "mstr"
	char *mstr = (char *)buff.c_str();

	// Parse it with StringTokenizer
	StringTokenizer tok(mstr);
	StringTokenizer::iterator it = tok.begin();

	// Find out the IP address that is attacked
	InetHostAddress attacked(*it);
	++it;
	// Find out if it's a flow-based or a packet-based attack
	int mode = atoi(*it);
	++it;
	// Find out the owner (email address) to which we must send an email
	// regarding the attack, if we are supposed to later on
	char *owner = (char *)(*it);

	// Get the logical output interface for the detector that connected to us
	uint16_t output = getOutput();
	TimeSlot *ts = cl->getProcTS();
	map< uint16_t, map <uint16_t, IfPair *> > *inttree = ts->getIntTree();
	map <uint16_t, IfPair *> nm = (*inttree)[output];

	for (map<uint16_t, IfPair *>::iterator it = nm.begin(); it != nm.end(); it++){
		InetHostAddress distatt = (*it).second->getTopTalker(mode);
		if (distatt == attacked){
			RemoteDetector remdet = cl->getRemotePeer((*it).second->getFromIP());
			if (remdet.remote == "0"){
				cout << "Attack from " << (*it).second->getFromIP() << " to " << attacked << ", but no remote detector found" << endl;
			}
			else {
				PeerConnect *pr = new PeerConnect(remdet, distatt, mode, owner);
				pr->Detach();
			}
		}
	}
}

uint16_t Client::getOutput(){
	// Traverse through the dist hash table and find the IP address of the interface
	// assosiated with the detector that provides the data

	TimeSlot *ts = cl->getProcTS();
	map< uint16_t, map <uint16_t, IfPair *> > *inttree = ts->getIntTree();

	for (map< uint16_t, map <uint16_t, IfPair *> >::iterator it = (*inttree).begin(); it != (*inttree).end(); it++)
        for (map<uint16_t, IfPair *>::iterator dit = (*it).second.begin(); dit != (*it).second.end(); dit++){
			RemoteDetector rmd = cl->getRemotePeer((*dit).second->getOutIP());
			if (rmd.remote == prevdetector)
				return (*it).first;
		}

	return 0;
}

