/***************************************************************************
                                 collector.h
                             -------------------
    begin                : Fri Feb 1 2002
    copyright            : (C) 2002 by Constantinos A. Kotsokalis
    email                : ckotso@grnet.gr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef COLLECTOR_H
#define COLLECTOR_H

/**Collects netflow data, parses it and stores it in memory.
  *@author Constantinos A. Kotsokalis
  */

class Collector;

#include <cc++/socket.h>

#ifdef  CCXX_NAMESPACES
	using namespace std;
	using namespace ost;
#endif

#include "mconfig.h"
#include "timestamp.h"
#include "timeslot.h"
#include "pdu.h"
#include "webbroker.h"
#include "detector.h"

typedef struct RemoteDetector {
	InetHostAddress local, remote;
	uint16_t remport;
} RemoteDetector;

class Collector {
public: 
	Collector();
	virtual ~Collector();
	/** Does all the work (binds local socket, creates new threads, accepts data, etc) */
	void BindAndLoop();
	/** Uses the configuration directives stored in argument */
	void setConfiguration(MConfig &);
	/** Sets up interface pairs information */
	void setupIfPairs(const char *);
	
	virtual void setifinfofile(const char * _newVal);
	virtual const char * getifinfofile();
	void setRemotePeer(const InetHostAddress&, const InetHostAddress&, const uint16_t&);
	RemoteDetector getRemotePeer(const InetHostAddress&);
	void makeSlots(uint32_t&, uint32_t&);
	TimeSlot *getProcTS() { return tsProc; }

private: // Private attributes
	/** Keeps the filename for the interface info file (created by the python script) */
	const char * ifinfofile;
	InetHostAddress router, websrv;
	Mutex *mtx;
	uint32_t SlotPeriod, slotcount;
	float increase, percentage;
	char *incharge, *execcmd;
	uint16_t port, peer_port, web_port;
	vector<RemoteDetector> RemotePeers;
	UDPSocket sock;
	TimeSlot *tsAdd, *tsProc, *tsPrev, *tsLast;
	PDU *datapacket;
	uint8_t buff[MAX_DATAGRAM_LEN];
	uint32_t lostpackets;
};

#endif
