/***************************************************************************
                                 common.h
                             -------------------
    begin                : Fri Feb 1 2002
    copyright            : (C) 2002 by Constantinos A. Kotsokalis
    email                : ckotso@grnet.gr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef COMMON_H
#define COMMON_H

#include <cc++/socket.h>

#ifdef  CCXX_NAMESPACES
using namespace std;
using namespace ost;
#endif

#ifdef PACKAGE
#undef PACKAGE
#endif

#ifdef VERSION
#undef VERSION
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif

// Type definitions for unchigned char/short/int
typedef unsigned int uint32_t;
typedef unsigned short int uint16_t;
typedef unsigned char uint8_t;

// Type definitions for V1/V5/V8 headers and bodies
typedef struct V1Header {
	uint16_t version;
	uint16_t count;
	uint32_t SysUptime;
	uint32_t unix_secs;
	uint32_t unix_nsecs;
} V1Header;

typedef struct V5Header {
	uint16_t version;
	uint16_t count;
	uint32_t SysUptime;
	uint32_t unix_secs;
	uint32_t unix_nsecs;
	uint32_t flow_sequence;
	uint8_t engine_type;
	uint8_t engine_id;
} V5Header;

typedef struct V8Header {
	uint16_t version;
	uint16_t count;
	uint32_t SysUptime;
	uint32_t unix_secs;
	uint32_t unix_nsecs;
	uint32_t flow_sequence;
	uint8_t engine_type;
	uint8_t engine_id;
	uint8_t aggregation;
	uint8_t agg_version;	
} V8Header;

typedef struct V1Body {
	uint32_t srcaddr;
	uint32_t dstaddr;
	uint32_t nexthop;
	uint16_t input;
	uint16_t output;
	uint32_t dPkts;
	uint32_t dOctets;
	uint32_t First;
	uint32_t Last;
	uint16_t srcport;
	uint16_t dstport;
	uint16_t pad;
	uint8_t prot;
	uint8_t tos;
	uint8_t tcp_flags;
	uint8_t pad2;
	uint16_t pad3;
	uint32_t reserved1;
	uint32_t reserved2;
} V1Body;

typedef struct V5Body {
	uint32_t srcaddr;
	uint32_t dstaddr;
	uint32_t nexthop;
	uint16_t input;
	uint16_t output;
	uint32_t dPkts;
	uint32_t dOctets;
	uint32_t First;
	uint32_t Last;
	uint16_t srcport;
	uint16_t dstport;
	uint8_t pad;
	uint8_t tcp_flags;
	uint8_t prot;
	uint8_t tos;
	uint16_t dst_as;
	uint16_t src_as;
	uint8_t dst_mask;
	uint8_t src_mask;
	uint16_t pad2;
} V5Body;

typedef struct V8BodyAS {
	uint32_t flows;
	uint32_t dPkts;
	uint32_t dOctets;
	uint32_t First;
	uint32_t Last;
	uint16_t src_as;
	uint16_t dst_as;
	uint16_t input;
	uint16_t output;
} V8BodyAS;

typedef struct V8BodyPP {
	uint32_t flows;
	uint32_t dPkts;
	uint32_t dOctets;
	uint32_t First;
	uint32_t Last;
	uint8_t prot;
	uint8_t pad;
	uint16_t reserved;
	uint16_t srcport;
	uint16_t dstport;
} V8BodyPP;

typedef struct V8BodySP {
	uint32_t flows;
	uint32_t dPkts;
	uint32_t dOctets;
	uint32_t First;
	uint32_t Last;
	uint32_t src_prefix;
	uint8_t src_mask;
	uint8_t pad;
	uint16_t src_as;
	uint16_t input;
} V8BodySP;

typedef struct V8BodyDP {
	uint32_t flows;
	uint32_t dPkts;
	uint32_t dOctets;
	uint32_t First;
	uint32_t Last;
	uint32_t dst_prefix;
	uint8_t dst_mask;
	uint8_t pad;
	uint16_t dst_as;
	uint16_t output;	
} V8BodyDP;

typedef struct V8BodyPR {
	uint32_t flows;
	uint32_t dPkts;
	uint32_t dOctets;
	uint32_t First;
	uint32_t Last;
	uint32_t src_prefix;
	uint32_t dst_prefix;
	uint8_t dst_mask;
	uint8_t src_mask;
	uint16_t resrerved;
	uint16_t src_as;
	uint16_t dst_as;
	uint16_t input;
	uint16_t output;
} V8BodyPR;

#define NETFLOW_V1 1
#define NETFLOW_V5 5
#define NETFLOW_V8 8

#define MAX(A, B) (A > B ? A : B)

#define V1_HEADER_LEN sizeof(V1Header)
#define V5_HEADER_LEN sizeof(V5Header)
#define V8_HEADER_LEN sizeof(V8Header)

#define V1_BODY_LEN sizeof(V1Body)
#define V5_BODY_LEN sizeof(V5Body)
#define V8AS_BODY_LEN sizeof(V8BodyAS)
#define V8PP_BODY_LEN sizeof(V8BodyPP)
#define V8SP_BODY_LEN sizeof(V8BodySP)
#define V8DP_BODY_LEN sizeof(V8BodyDP)
#define V8PR_BODY_LEN sizeof(V8BodyPR)

#define MAXBODY1 MAX(V1_BODY_LEN, V5_BODY_LEN)
#define MAXBODY2 MAX(MAXBODY1, V8AS_BODY_LEN)
#define MAXBODY3 MAX(MAXBODY2, V8PP_BODY_LEN)
#define MAXBODY4 MAX(MAXBODY3, V8SP_BODY_LEN)
#define MAXBODY5 MAX(MAXBODY4, V8DP_BODY_LEN)

#define MAX_FLOW_BODY MAX(MAXBODY5, V8PR_BODY_LEN)

#define V1_MAX_FLOWS 25
#define V5_MAX_FLOWS 30
#define V8AS_MAX_FLOWS 51
#define V8PP_MAX_FLOWS 51
#define V8SP_MAX_FLOWS 44
#define V8DP_MAX_FLOWS 44
#define V8PR_MAX_FLOWS 35

#define MAXFL1 MAX(V1_MAX_FLOWS, V5_MAX_FLOWS)
#define MAXFL2 MAX(MAXFL1, V8AS_MAX_FLOWS)
#define MAXFL3 MAX(MAXFL2, V8PP_MAX_FLOWS)
#define MAXFL4 MAX(MAXFL3, V8SP_MAX_FLOWS)
#define MAXFL5 MAX(MAXFL4, V8DP_MAX_FLOWS)

#define MAX_FLOWS MAX(MAXFL5, V8PR_MAX_FLOWS)

#define V1_MAX_DG_SIZE (V1_HEADER_LEN + V1_MAX_FLOWS * V1_BODY_LEN)
#define V5_MAX_DG_SIZE (V5_HEADER_LEN + V5_MAX_FLOWS * V5_BODY_LEN)
#define V8AS_MAX_DG_SIZE (V8_HEADER_LEN + V8AS_MAX_FLOWS * V8AS_BODY_LEN)
#define V8PP_MAX_DG_SIZE (V8_HEADER_LEN + V8PP_MAX_FLOWS * V8PP_BODY_LEN)
#define V8SP_MAX_DG_SIZE (V8_HEADER_LEN + V8SP_MAX_FLOWS * V8SP_BODY_LEN)
#define V8DP_MAX_DG_SIZE (V8_HEADER_LEN + V8DP_MAX_FLOWS * V8DP_BODY_LEN)
#define V8PR_MAX_DG_SIZE (V8_HEADER_LEN + V8PR_MAX_FLOWS * V8PR_BODY_LEN)

#define MAX8_1 MAX(V8AS_MAX_DG_SIZE, V8PP_MAX_DG_SIZE)
#define MAX8_2 MAX(MAX8_1, V8SP_MAX_DG_SIZE)
#define MAX8_3 MAX(MAX8_2, V8DP_MAX_DG_SIZE)
#define MAX8 MAX(MAX8_3, V8PR_MAX_DG_SIZE)
#define MAX1_5 MAX(V1_MAX_DG_SIZE, V5_MAX_DG_SIZE)

#define MAX_DATAGRAM_LEN MAX(MAX1_5, MAX8)

#define AGGR_AS 0x01
#define AGGR_PP 0x02
#define AGGR_SP 0x03
#define AGGR_DP 0x04
#define AGGR_PR 0x05

#define AS_ISSET(x) (AGGR_AS == x)
#define PP_ISSET(x) (AGGR_PP == x)
#define SP_ISSET(x) (AGGR_SP == x)
#define DP_ISSET(x) (AGGR_DP == x)
#define PR_ISSET(x) (AGGR_PR == x)

// Constants for the getTopTalker() method
#define TALK_FLOWS 1
#define TALK_PACKETS 2

#endif
