/***************************************************************************
                                 detector.cpp
                             -------------------
    begin                : Mon Feb 4 2002
    copyright            : (C) 2002 by Constantinos A. Kotsokalis
    email                : ckotso@grnet.gr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "detector.h"

Detector::Detector(TimeSlot *ts, Collector *collector, float& percent, float& inc, string lowner, string execcmd, Mutex *mutex){
	procts = ts;
	cl = collector;
	percentage = percent;
	increase = inc;
	owner = lowner;
	cmd = execcmd;
	mtx = mutex;
}

Detector::~Detector() {

}

void Detector::Final() {
	delete this;
}

void Detector::Run(){
	IfPair *curif, *previf;

	mtx->EnterMutex();

	uint32_t totalflows = procts->getFlows();
	uint32_t totalpackets = procts->getPackets();
	//uint32_t totaloctets = procts->getOctets();

	map< uint16_t, map <uint16_t, IfPair *> > *IntTree = procts->getIntTree();

	for (map< uint16_t, map<uint16_t, IfPair *> >::iterator it = (*IntTree).begin(); it != (*IntTree).end(); it++){
		for (map<uint16_t, IfPair *>::iterator sit = (*it).second.begin(); sit != (*it).second.end(); sit++){
			curif = (*sit).second;
			previf = NULL;
			TimeSlot *previous = procts->getPrevious();
			if (previous){
				map< uint16_t, map<uint16_t, IfPair *> > *previnttree = previous->getIntTree();
				if ((*previnttree).count(curif->getoutput()))
					if ((*previnttree).count(curif->getinput()))
						previf = (*previnttree)[curif->getoutput()][curif->getinput()];
			}

			if (!previf || !previf->getflows() || !previf->getpackets())
				continue;

			uint32_t curflows = curif->getflows();
			uint32_t prevflows = previf->getflows();
 			uint32_t curpacks = curif->getpackets();
			uint32_t prevpacks = previf->getpackets();
						
			bool flattack = (((((float)curflows - (float)prevflows) / (float)prevflows) > increase) && (((float)curflows / (float)totalflows) > percentage));
			bool pkattack = (((((float)curpacks - (float)prevpacks) / (float)prevpacks) > increase) && (((float)curpacks / (float)totalpackets) > percentage));

			if (flattack || pkattack){
				curif->setflows(prevflows);
				curif->setpackets(prevpacks);
				if (!previf->IsDos()){
					if (!previf->MarkedDos())
						curif->MarkedDos(true);
					else {
						// else (if we already have an indication)
						int mode;
						// check the attack's type (flow or packet based_
						if (flattack)
							mode = TALK_FLOWS;
						else
							mode = TALK_PACKETS;
						// Log the attack
						InetHostAddress ia = curif->getTopTalker(mode);
						InetHostAddress local = curif->getFromIP();
						LogAttackStart(curif->getinput(), curif->getoutput(), local, ia, curflows, curpacks);
						// Mark the pair as under attack
						curif->MarkedDos(false);
						curif->IsDos(true);
						PeerConnect *pr = new PeerConnect(cl->getRemotePeer(local), ia, mode, (char *)owner.c_str());
						pr->Detach();
					}
				}
				else {
					curif->MarkedDos(false);
					curif->IsDos(true);
				}
			}
			else {
				// If we know an attack takes place, but it seems to have stopped,
				// log its finish and mark the IsDos flag as false
				if (previf->IsDos()){
					LogAttackStop(curif->getinput(), curif->getoutput(), curif->getFromIP(), curflows, curpacks);
					curif->IsDos(false);
				}
				// Also mark the MarkedDos flag (indicating attack suspicion) as false
				curif->MarkedDos(false);
			}
		}
	}

	mtx->LeaveMutex();
}

void Detector::LogAttackStart(uint16_t inp, uint16_t outp, InetHostAddress srcif, InetHostAddress dst, uint32_t fl, uint32_t pk){
	char str[1024];
	struct in_addr in, out;

	memset(str, '\0', 1024);
	in = srcif.getAddress();
	out = dst.getAddress();
	sprintf(str, "%s START %d %d ", cmd.c_str(), inp, outp);
	string sstr = string(str);
	sstr = sstr + string(" ") + string(inet_ntoa(in)) + \
			string(" ") + string(inet_ntoa(out));
	sprintf(str, " %d %d", fl, pk);
	sstr = sstr + string(str);
	system(sstr.c_str());
}

void Detector::LogAttackStop(uint16_t inp, uint16_t outp, InetHostAddress srcif, uint32_t fl, uint32_t pk){
	char str[1024];
	struct in_addr in;

	memset(str, '\0', 1024);
	in = srcif.getAddress();
	sprintf(str, "%s STOP %d %d ", cmd.c_str(), inp, outp);
	string sstr = string(str);
	sstr = sstr + string(" ") + string(inet_ntoa(in));
	sprintf(str, " %d %d", fl, pk);
	sstr = sstr + string(str);
	system(sstr.c_str());
}
