/***************************************************************************
                                 flow.cpp
                             -------------------
    begin                : Fri Feb 1 2002
    copyright            : (C) 2002 by Constantinos A. Kotsokalis
    email                : ckotso@grnet.gr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "flow.h"

Flow::Flow(){

}

Flow::~Flow(){

}

// Set the flow's data from the buffer passwd as an argument
void Flow::flSet(uint16_t& v, uint32_t& curtime, uint32_t& usec, uint32_t& umsec, uint8_t *buff, uint8_t maggtype=0){
	// NetFlow version information
	version = v;
	// v8 aggregation type
	aggtype = maggtype;

	switch (version){
		// For NetFlow v1...
		case NETFLOW_V1:
				memcpy((void *)&(flowbody.v1), (void *)buff, V1_BODY_LEN);
				flowbody.v1.input = ntohs(flowbody.v1.input);
				flowbody.v1.output = ntohs(flowbody.v1.output);
				input = flowbody.v1.input;
				output = flowbody.v1.output;
				dstaddr = ntohl(flowbody.v1.dstaddr);
				break;
		// For NetFlow v5...
		case NETFLOW_V5:
				memcpy((void *)&(flowbody.v5), (void *)buff, V5_BODY_LEN);
				flowbody.v5.input = ntohs(flowbody.v5.input);
				flowbody.v5.output = ntohs(flowbody.v5.output);
				input = flowbody.v5.input;
				output = flowbody.v5.output;
				dstaddr = ntohl(flowbody.v5.dstaddr);
				break;
		// For NetFlow v8. Commented lines include data that is not found in
		// respective aggregation types, but they are left there as reminders
		// of what's missing in each type.
		case NETFLOW_V8:
				if (AS_ISSET(aggtype)){
					memcpy((void *)&(flowbody.v8as), (void *)buff, V8AS_BODY_LEN);
					input = ntohs(flowbody.v8as.input);
					output = ntohs(flowbody.v8as.output);
					//dstaddr = flowbody.v8as.dstaddr;
				}
				else if (PP_ISSET(aggtype)){
					memcpy((void *)&(flowbody.v8pp), (void *)buff, V8PP_BODY_LEN);
					//input = flowbody.v8pp.input;
					//output = flowbody.v8pp.output;
					//dstaddr = flowbody.v8pp.dstaddr;
				}
				else if (SP_ISSET(aggtype)){
					memcpy((void *)&(flowbody.v8sp), (void *)buff, V8SP_BODY_LEN);
					input =ntohs( flowbody.v8sp.input);
					//output = flowbody.v8sp.output;
					//dstaddr = flowbody.v8sp.dstaddr;
				}
				else if (DP_ISSET(aggtype)){
					memcpy((void *)&(flowbody.v8dp), (void *)buff, V8DP_BODY_LEN);
					//input = flowbody.v8dp.input;
					output = ntohs(flowbody.v8dp.output);
					dstaddr = ntohl(flowbody.v8dp.dst_prefix) | flowbody.v8dp.dst_mask;
				}
				else if (PR_ISSET(aggtype)){
					memcpy((void *)&(flowbody.v8pr), (void *)buff, V8PR_BODY_LEN);
					input = ntohs(flowbody.v8pr.input);
					output = ntohs(flowbody.v8pr.output);
					dstaddr = ntohl(flowbody.v8dp.dst_prefix) | flowbody.v8dp.dst_mask;
				}
				break;
	}

	uint32_t fm = curtime - getStartTime();
	uint32_t lm = curtime - getEndTime();
	
	uint32_t utime_begin_sec = usec - fm / 1000;
	uint32_t utime_end_sec = usec - lm / 1000;
	
	uint32_t utime_begin_msec, utime_end_msec;

	if (fm % 1000 > umsec){
		utime_begin_msec = fm % 1000 - umsec;
		utime_begin_sec = utime_begin_sec - 1;
	}
	else
		utime_begin_msec = umsec - fm % 1000;

	if (lm % 1000 > umsec){
		utime_end_msec = lm % 1000 - umsec;
		utime_end_sec = utime_end_sec - 1;
	}
	else
		utime_end_msec = umsec - lm % 1000;
	
	begin = TimeStamp(utime_begin_sec, utime_begin_msec);
	end = TimeStamp(utime_end_sec, utime_end_msec);
}

// Get the number of packets for a flow
const uint32_t Flow::getPackets(){
	switch (version){
		case 1:
				return ntohl(flowbody.v1.dPkts);
		case 5:
				return ntohl(flowbody.v5.dPkts);
		case 8:
				if (AS_ISSET(aggtype))
					return ntohl(flowbody.v8as.dPkts);
				else if (PP_ISSET(aggtype))
					return ntohl(flowbody.v8pp.dPkts);
				else if (SP_ISSET(aggtype))
					return ntohl(flowbody.v8sp.dPkts);
				else if (DP_ISSET(aggtype))
					return ntohl(flowbody.v8dp.dPkts);
				else if (PR_ISSET(aggtype))
					return ntohl(flowbody.v8pr.dPkts);
		default:
				return 0;
	}
}

// Get the number of octets for a flow
const uint32_t Flow::getOctets(){
	switch (version){
		case 1:
				return ntohl(flowbody.v1.dOctets);
		case 5:
				return ntohl(flowbody.v5.dOctets);
		case 8:
				if (AS_ISSET(aggtype))
					return ntohl(flowbody.v8as.dOctets);
				else if (PP_ISSET(aggtype))
					return ntohl(flowbody.v8pp.dOctets);
				else if (SP_ISSET(aggtype))
					return ntohl(flowbody.v8sp.dOctets);
				else if (DP_ISSET(aggtype))
					return ntohl(flowbody.v8dp.dOctets);
				else if (PR_ISSET(aggtype))
					return ntohl(flowbody.v8pr.dOctets);
		default:
				return 0;
	}
}

// Get the timestamp denoting the beginning of a flow
const uint32_t Flow::getStartTime(){
	switch (version){
		case 1:
				return ntohl(flowbody.v1.First);
		case 5:
				return ntohl(flowbody.v5.First);
		case 8:
				if (AS_ISSET(aggtype))
					return ntohl(flowbody.v8as.First);
				else if (PP_ISSET(aggtype))
					return ntohl(flowbody.v8pp.First);
				else if (SP_ISSET(aggtype))
					return ntohl(flowbody.v8sp.First);
				else if (DP_ISSET(aggtype))
					return ntohl(flowbody.v8dp.First);
				else if (PR_ISSET(aggtype))
					return ntohl(flowbody.v8pr.First);
		default:
				return 0;
	}
}

// Get the timestamp denoting the end of a flow
const uint32_t Flow::getEndTime(){
	switch (version){
		case 1:
				return ntohl(flowbody.v1.Last);
		case 5:
				return ntohl(flowbody.v5.Last);
		case 8:
				if (AS_ISSET(aggtype))
					return ntohl(flowbody.v8as.Last);
				else if (PP_ISSET(aggtype))
					return ntohl(flowbody.v8pp.Last);
				else if (SP_ISSET(aggtype))
					return ntohl(flowbody.v8sp.Last);
				else if (DP_ISSET(aggtype))
					return ntohl(flowbody.v8dp.Last);
				else if (PR_ISSET(aggtype))
					return ntohl(flowbody.v8pr.Last);
		default:
				return 0;
	}
}

