/***************************************************************************
                                 flow.h
                             -------------------
    begin                : Fri Feb 1 2002
    copyright            : (C) 2002 by Constantinos A. Kotsokalis
    email                : ckotso@grnet.gr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FLOW_H
#define FLOW_H

/**Represents a unique flow of data.
  *@author Constantinos A. Kotsokalis
  */

class Flow;

#include "common.h"
#include "timestamp.h"

// A union of the possible flow body formats
union FlowBody {
	V1Body v1;
	V5Body v5;
	V8BodyAS v8as;
	V8BodyPP v8pp;
	V8BodySP v8sp;
	V8BodyDP v8dp;
	V8BodyPR v8pr;
};

// Represents a single flow of data as provided by NetFlow
class Flow {
private:
	// The flow body
	FlowBody flowbody;
	// Protocol version
	uint16_t version;
	// Input and output interface
	uint16_t input, output;
	// Destination address
	uint32_t dstaddr;
	// Aggregation type
	uint8_t aggtype;
	TimeStamp begin, end;
	
public:
	Flow();
	virtual ~Flow();
	// Set the flow data from a raw buffer
	void flSet(uint16_t&, uint32_t&, uint32_t&, uint32_t&, uint8_t *, uint8_t maggtype=0);
	// Return the input interface
	virtual const uint16_t getInput() {return input;}
	// Return the output interface
	virtual const uint16_t getOutput() {return output;}
	// Return the version number
	virtual const uint16_t getVersion() {return version;}
	// Return the destination address
	virtual const uint32_t getDstAddr() {return dstaddr;}
	// Return the number of packets thath this flow holds
	virtual const uint32_t getPackets();
	// Return the number of octets that this flow holds
	virtual const uint32_t getOctets();
	// Return the timestamp indicating when the flow started
	virtual const uint32_t getStartTime();
	// Return the timestamp indicating when the flow ended
	virtual const uint32_t getEndTime();
	virtual TimeStamp getUBegin() { return begin; }
	virtual TimeStamp getUEnd() { return end; }
};

#endif
