/***************************************************************************
                                 ifpair.cpp
                             -------------------
    begin                : Fri Feb 1 2002
    copyright            : (C) 2002 by Constantinos A. Kotsokalis
    email                : ckotso@grnet.gr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ifpair.h"

IfPair::IfPair(uint16_t& i, uint16_t& o){
	input = i;
	output = o;
	marked_dos = is_dos = false;
	flows = packets = octets = 0;
}

IfPair::~IfPair(){

}

const uint16_t& IfPair::getinput(){
	return input;
}

const uint16_t& IfPair::getoutput(){
	return output;
}

void IfPair::setinput( const uint16_t& _newVal){
	input = _newVal;
}

void IfPair::setoutput( const uint16_t& _newVal){
	output = _newVal;
}

const uint32_t& IfPair::getflows(){
	return flows;
}

const uint32_t& IfPair::getpackets(){
	return packets;
}

const uint32_t& IfPair::getoctets(){
	return octets;
}

void IfPair::Add(const uint32_t& pk, const uint32_t& oct, const uint32_t& pdst_addr){
	uint32_t dst_addr;

	// Add amount of packets and octets (plus one flow) to the timeslot
	flows += 1;
	packets += pk;
	octets += oct;

	// Add data to the IP address tree for the destination address.
	// This data is used by getTopTalker to find out the destination address
	// that the largest amount of traffic heads to.
	dst_addr = pdst_addr;
	uint8_t b1 = dst_addr >> 24;
	uint8_t b2 = (dst_addr << 8) >> 24;
	uint8_t b3 = (dst_addr << 16) >> 24;
	uint8_t b4 = (dst_addr << 24) >> 24;
	iptree[b1][b2][b3][b4].flows += 1;
	iptree[b1][b2][b3][b4].packets += pk;
	iptree[b1][b2][b3][b4].octets += oct;
}

InetHostAddress IfPair::getTopTalker(int mode){
	if (mode == TALK_FLOWS)
		return fl_getTopTalker();
	else
		return pk_getTopTalker();
}

InetHostAddress IfPair::fl_getTopTalker(){
    uint32_t curfl=0;
    uint32_t b1=0, b2=0, b3=0, b4=0;

    for (map<uint8_t, level2>::iterator it1 = iptree.begin(); it1 != iptree.end(); it1++)
        for (map<uint8_t, level3>::iterator it2 = (*it1).second.begin(); it2 !=(*it1).second.end(); it2++)
            for (map<uint8_t, level4>::iterator it3 = (*it2).second.begin(); it3 != (*it2).second.end(); it3++)
                for (map<uint8_t, NetNode>::iterator it4 = (*it3).second.begin(); it4 != (*it3).second.end(); it4++)
                    if ((*it4).second.flows > curfl){
                        b1 = (uint32_t)(*it1).first;
                        b2 = (uint32_t)(*it2).first;
                        b3 = (uint32_t)(*it3).first;
                        b4 = (uint32_t)(*it4).first;
                        curfl = (*it4).second.flows;
                    }

    uint32_t addr = htonl((b1 << 24) + (b2 << 16) + (b3 << 8) + b4);
    struct in_addr in;
    memcpy((void *)&(in.s_addr), (void *)&addr, sizeof(addr));
    InetHostAddress retval(in);

    return retval;
}

InetHostAddress IfPair::pk_getTopTalker(){
    uint32_t curpk=0;
    uint32_t b1=0, b2=0, b3=0, b4=0;

    // Again, find the correct leaf
    for (map<uint8_t, level2>::iterator it1 = iptree.begin(); it1 != iptree.end(); it1++)
        for (map<uint8_t, level3>::iterator it2 = (*it1).second.begin(); it2 !=(*it1).second.end(); it2++)
            for (map<uint8_t, level4>::iterator it3 = (*it2).second.begin(); it3 != (*it2).second.end(); it3++)
                for (map<uint8_t, NetNode>::iterator it4 = (*it3).second.begin(); it4 != (*it3).second.end(); it4++)
                    if ((*it4).second.packets > curpk){
                        b1 = (uint32_t)(*it1).first;
                        b2 = (uint32_t)(*it2).first;
                        b3 = (uint32_t)(*it3).first;
                        b4 = (uint32_t)(*it4).first;
                        curpk = (*it4).second.packets;
                    }

    uint32_t addr = htonl((b1 << 24) + (b2 << 16) + (b3 << 8) + b4);
    struct in_addr in;
    memcpy((void *)&(in.s_addr), (void *)&addr, sizeof(addr));
    InetHostAddress retval(in);

    return retval;
}

uint32_t& IfPair::getflows(InetHostAddress& ia){
	struct in_addr in = ia.getAddress();
	uint32_t addr = ntohl(in.s_addr);
	uint8_t b1 = addr >> 24;
	uint8_t b2 = (addr << 8) >> 24;
	uint8_t b3 = (addr << 16) >> 24;
	uint8_t b4 = (addr << 24) >> 24;

	return iptree[b1][b2][b3][b4].flows;
}

uint32_t& IfPair::getpackets(InetHostAddress& ia){
	struct in_addr in = ia.getAddress();
	uint32_t addr = ntohl(in.s_addr);
	uint8_t b1 = addr >> 24;
	uint8_t b2 = (addr << 8) >> 24;
	uint8_t b3 = (addr << 16) >> 24;
	uint8_t b4 = (addr << 24) >> 24;

	return iptree[b1][b2][b3][b4].packets;
}
