/***************************************************************************
                                 ifpair.h
                             -------------------
    begin                : Fri Feb 1 2002
    copyright            : (C) 2002 by Constantinos A. Kotsokalis
    email                : ckotso@grnet.gr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef IFPAIR_H
#define IFPAIR_H

/**Represents the traffic between two different
interfaces.
  *@author Constantinos A. Kotsokalis
  */

class IfPair;

#include "common.h"

#include <map>

// Represents an IP addres to which at least one flow headed
class NetNode {
public:
	NetNode() { flows = packets = octets = 0; }
	virtual ~NetNode() {;}
	virtual const uint32_t& getflows() { return flows; }
	virtual const uint32_t& getpackets() { return packets; }
	virtual const uint32_t& getoctets() { return octets; }

private:
	uint32_t flows, packets, octets;
	
friend class IfPair;
};

// The next line represents a tree with all the addresses to which
// traffic headed
typedef map<uint8_t, NetNode> level4;
typedef map<uint8_t, level4> level3;
typedef map<uint8_t, level3> level2;
typedef map<uint8_t, level2> IPTree;

class IfPair {
public:
	IfPair(uint16_t&, uint16_t&);
	virtual ~IfPair();
  	
  	virtual const uint16_t& getinput();  	
  	virtual void setinput( const uint16_t& _newVal);
  	virtual const uint16_t& getoutput();
	virtual void setoutput( const uint16_t& _newVal);
	
	virtual const uint32_t& getflows();
	virtual const uint32_t& getpackets();
	virtual const uint32_t& getoctets();

	virtual uint32_t& getflows(InetHostAddress&);
	virtual uint32_t& getpackets(InetHostAddress&);
		
	virtual void setflows(uint32_t fl) { flows = fl; }
	virtual void setpackets(uint32_t pk) { packets = pk; }
	virtual void setoctets(uint32_t oct) { octets = oct; }

	virtual void setFromIP(const char *s){ InputIP = InetHostAddress(s); }
	virtual void setOutIP(const char *s){ OutputIP = InetHostAddress(s); }
	virtual InetHostAddress& getFromIP() { return InputIP; }
	virtual InetHostAddress& getOutIP() { return OutputIP; }
	
	virtual void Add(const uint32_t&, const uint32_t&, const uint32_t&);

	virtual bool MarkedDos() {return marked_dos;}
	// Set potential DoS attack
	virtual void MarkedDos(bool b) {marked_dos = b;}
	// Return whether an attack is going on for this pair
	virtual bool IsDos() {return is_dos;}
	// Set the flag to indicate an attack is going on
	virtual void IsDos(bool b) {is_dos = b;}
	
	virtual InetHostAddress getTopTalker(int);
	virtual InetHostAddress fl_getTopTalker();
	virtual InetHostAddress pk_getTopTalker();

private: // Private attributes
	IPTree iptree;
	uint16_t input, output;
	uint32_t flows, packets, octets;
	bool marked_dos, is_dos;
	InetHostAddress InputIP, OutputIP;
};

#endif
