/***************************************************************************
                                 main.cpp
                             -------------------
    begin                : Fri Feb  1 11:58:20 EET 2002
    copyright            : (C) 2002 by Constantinos A. Kotsokalis
    email                : ckotso@grnet.gr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "mconfig.h"
#include "collector.h"

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include <stdlib.h>

void HelpMsgExit(char *argv[]);

int main(int argc, char *argv[])
{
	string config, speeds;
	char c;

	// Get argument options
	while ((c = getopt(argc, argv, "c:s:h")) != EOF)
		switch (c) {
			case 'c':
					config = string(optarg);
					break;
			case 's':
					speeds = string(optarg);
					break;
			default:
				HelpMsgExit(argv);
	}

	// Both config file and interface description file must be present
	if (config.empty() || speeds.empty())
		HelpMsgExit(argv);

	const char *charconfig = config.c_str();
	const char *charspeeds = speeds.c_str();

	// Create a collector instance (the main component of the application
	Collector collector = Collector();
	// Set the ifinfofile variable
	collector.setifinfofile(charspeeds);
	// Read in the configuration
	MConfig mconfig = MConfig(charconfig);
	// Set collector options from the configuration data
	collector.setConfiguration(mconfig);
	// Start the collector
	collector.BindAndLoop();
}

// A startup error message
void HelpMsgExit(char *argv[]){
	cout << "Usage: " << argv[0] << " <args>" << endl;
	cout << "Args:  -c <config filename> (Compulsory argument)" << endl;
	cout << "       -s <interface bandwidth file generated by speed.py> (Compulsory agument)" << endl;
	cout << "       -h (Prints this help message)" << endl;
	::exit(0);
}
