/***************************************************************************
                                 pdu.h
                             -------------------
    begin                : Fri Feb 1 2002
    copyright            : (C) 2002 by Constantinos A. Kotsokalis
    email                : ckotso@grnet.gr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PDU_H
#define PDU_H

/**Represents a netflow PDU.
  *@author Constantinos A. Kotsokalis
  */

class PDU;

#include "common.h"
#include "flow.h"
#include "timeslot.h"

#include <map>

// The datapacket's header (any version)
union FlowHeader {
	V1Header v1;
	V5Header v5;
	V8Header v8;
};

class PDU {
private:
	FlowHeader flowheader;
	// An array of flows, as provided by the datagram sent to us by the router
	Flow flows[MAX_FLOWS];
	uint16_t version;
	uint16_t numflows;
	uint32_t usec, umsec, sysuptime;
	// The datagram body
	uint8_t flowbody[MAX_FLOW_BODY];

public:
	// Constructor
	PDU();
	// Destructor
	virtual ~PDU();
	// Method that sets the datapacket data from the raw datagram passed as an argument
	virtual void dpSet(uint16_t&, uint16_t&, uint32_t&, uint32_t&, uint32_t&, uint8_t *, uint8_t aggtype=0);
	// Method that adds all flows to a timeslot
	virtual void addFlows(TimeSlot *);
};

#endif
