/***************************************************************************
                          peer.cpp  -  description
                             -------------------
    begin                : Fri Nov 2 2001
    copyright            : (C) 2001 by Constantinos A. Kotsokalis
    email                : ckotso@netmode.ntua.gr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "peer.h"

// Create a thread to bind on a local port and wait for incoming connections from
// other (neighbour) detectors
Peer::Peer(uint16_t lport, Collector *collector){
	cl = collector;
	port = lport;
	// Detach from parent thread
	Detach();
}

Peer::~Peer(){

}

void Peer::Final() {
	delete this;
}

class peerTCPSocket : public TCPSocket {
private:
	Collector *cl;

protected:
	bool OnAccept(const InetHostAddress &ia, tpport_t port);

public:
	peerTCPSocket(InetAddress &ia, uint16_t port) : TCPSocket(ia, port) {};

friend Peer;
};

bool peerTCPSocket::OnAccept(const InetHostAddress &ia, tpport_t port)
{
	TimeSlot *ts = cl->getProcTS();
	map< uint16_t, map <uint16_t, IfPair *> > *inttree = ts->getIntTree();
	// Search for the remote address in our list of known detectors
	for (map< uint16_t, map <uint16_t, IfPair *> >::iterator it = (*inttree).begin(); it != (*inttree).end(); it++)
        for (map<uint16_t, IfPair *>::iterator dit = (*it).second.begin(); dit != (*it).second.end(); dit++){
			RemoteDetector rmd = cl->getRemotePeer((*dit).second->getOutIP());
			if (rmd.remote == ia)
				return true;
		}

	return false;
}

void Peer::Run(){
	InetHostAddress next;
	Client *ts;
	peerTCPSocket *sock;

	try {
		InetAddress addr = "0";
		sock = new peerTCPSocket(addr, port);
		sock->cl = cl;
	}
	catch (Socket *socket){
		perror("Peer socket could not be created");
		return;
	}

	while (1)
		// If there is a new connection waiting...
		if (sock->isPendingConnection()){
			try {
				ts = new Client(*sock, next, cl);
			}
			catch (Socket *socket) {
				perror("Cannot spawn client thread");
				continue;
			}
		}
}

