/***************************************************************************
                                 timeslot.cpp
                             -------------------
    begin                : Fri Feb 1 2002
    copyright            : (C) 2002 by Constantinos A. Kotsokalis
    email                : ckotso@grnet.gr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "timeslot.h"

TimeSlot::TimeSlot(TimeStamp& ts_begin, uint32_t& period, const char *ifinfo){
	flows = octets = packets = 0;
	previous = NULL;
	begin = ts_begin;
	uint32_t endsec = begin.getSec() + period;
	uint32_t endmsec = begin.getMsec() - 1;
	end = TimeStamp(endsec, endmsec);
	setupIfPairs(ifinfo);
}

TimeSlot::~TimeSlot(){
	for (map< uint16_t, map<uint16_t, IfPair*> >::iterator fit = inttree.begin(); fit != inttree.end(); fit++)
		for (map<uint16_t, IfPair*>::iterator sit = (*fit).second.begin(); sit != (*fit).second.end(); sit++)
			delete (*sit).second;
}

void TimeSlot::Add(Flow& fl){
	uint16_t input = fl.getInput();
	uint16_t output = fl.getOutput();

	if (inttree.count(output))
		if (inttree[output].count(input)){
			uint32_t pknum = fl.getPackets();
			uint32_t octnum = fl.getOctets();
			uint32_t dst_addr = fl.getDstAddr();

			TimeStamp flbegin = fl.getUBegin();
			TimeStamp flend = fl.getUEnd();
			
			float percent = 0;

			if (flend > begin){
				if (flbegin >= begin)
					percent = 1;
				else {
					TimeStamp diff = flend - begin;
					uint32_t msec = diff.getSec() * 1000 + diff.getMsec();
					TimeStamp duration = flend - flbegin;
					uint32_t flmsec = duration.getSec() * 1000 + duration.getMsec();
					percent = (float)msec / (float)flmsec;
				}
			}
			else if (previousadd)
				previousadd->Add(fl);

			uint32_t pkadd = (uint32_t)((float)pknum * percent);
			uint32_t octadd = (uint32_t)((float)octnum * percent);
			
			if (percent){
				flows += 1;
				packets += pkadd;
				octets += octadd;
				inttree[output][input]->Add(pkadd, octadd, dst_addr);
			}
		}
}

/** Sets up interface pairs information */
void TimeSlot::setupIfPairs(const char *ifinfo){
	// Open the file created by speeds.py
	ifstream sf(ifinfo);
	// Exit if something went wrong
	if (!sf.is_open()){
		perror(ifinfo);
		exit(errno);
	}
	string line;
	vector<string> lines;
	// Insert all lines of the file in the "lines" vector
	while (getline(sf, line))
		lines.push_back(line);
	sf.close();

	// Now go through the vector to create all possible interface pairs (input/output)
	for (vector<string>::iterator firstit = lines.begin(); firstit != lines.end(); firstit++)
		for (vector<string>::iterator secit = lines.begin(); secit != lines.end(); secit++){
			const char *str1 = (*firstit).c_str();
			const char *str2 = (*secit).c_str();

			StringTokenizer tok1(str1, " \t");
			StringTokenizer::iterator it1 = tok1.begin();
			StringTokenizer tok2(str2, " \t");
			StringTokenizer::iterator it2 = tok2.begin();

			int input = atoi(*it1);
			int output = atoi(*it2);

			// If we are on the same line of the file, then continue
			if (input == output)
				continue;

			// Create a new Distribution object
			IfPair *ifpair = new IfPair((uint16_t)input, (uint16_t)output);
			if (!ifpair){
				perror("new");
				exit(errno);
			}
			++it1; ++it2;
			ifpair->setFromIP(*it1);
			ifpair->setOutIP(*it2);

			// Add the new object to a vector of the distribution hash table
			inttree[output][input] = ifpair;
		}
}
