/***************************************************************************
                                 timeslot.h
                             -------------------
    begin                : Fri Feb 1 2002
    copyright            : (C) 2002 by Constantinos A. Kotsokalis
    email                : ckotso@grnet.gr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TIMESLOT_H
#define TIMESLOT_H

/**Represents the traffic for a slot of time.
  *@author Constantinos A. Kotsokalis
  */

class TimeSlot;

#include <cc++/misc.h>

#ifdef  CCXX_NAMESPACES
	using namespace std;
	using namespace ost;
#endif

#include "common.h"
#include "ifpair.h"
#include "flow.h"
#include "timestamp.h"

#include <stdio.h>
#include <errno.h>

#include <iostream>
#include <fstream>

#include <string>
#include <vector>
#include <map>

class TimeSlot {
public: // Public attributes
	TimeSlot(TimeStamp&, uint32_t&, const char *);
	virtual ~TimeSlot();
	virtual void setupIfPairs(const char *);
	virtual void Add(Flow&);
	virtual void setPrevious(TimeSlot *ts) { previous = ts; }
	virtual void setPreviousAdd(TimeSlot *ts) { previousadd = ts; }
	TimeSlot *getPrevious() { return previous; }
	TimeStamp const getBegin() { return begin; }
	TimeStamp const getEnd() { return end; }
	map< uint16_t, map <uint16_t, IfPair *> > *getIntTree() { return &inttree; }
	uint32_t& getFlows() { return flows; }
	uint32_t& getPackets() { return packets; }
	uint32_t& getOctets() { return octets; }

private: // Private attributes
	map< uint16_t, map <uint16_t, IfPair *> > inttree;
	TimeStamp begin, end;
	TimeSlot *previous, *previousadd;
	uint32_t flows, octets, packets;
};

#endif
