/***************************************************************************
                                 webbroker.cpp
                             -------------------
    begin                : Fri Feb 1 2002
    copyright            : (C) 2002 by Constantinos A. Kotsokalis
    email                : ckotso@grnet.gr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "webbroker.h"

bool webTCPSocket::OnAccept(const InetHostAddress &ia, tpport_t port)
{
	if (ia != websrv){
		return false;
	}
	else
		return true;
}

WebBroker::WebBroker(uint16_t& lport, InetHostAddress& lwebsrv, Mutex *mutex, Collector *cl){
	port = lport;
	websrv = lwebsrv;
	mtx = mutex;
	collector = cl;
}

WebBroker::~WebBroker(){

}

void WebBroker::Run(){
	InetHostAddress next;
	webTCPSocket *sock;
	char line[255];

	try {
		InetAddress addr = "0";
		sock = new webTCPSocket(addr, port);
		sock->websrv = websrv;
	}
	catch (Socket *socket){
		perror("Web serving socket could not be created");
		return;
	}

	while (1)
		// If there is a new connection waiting...
		if (sock->isPendingConnection()){
			string buff;
			tcpstream tcp;
			tcp.open(*sock);

			// If OnAccept() returns false the socket is closed by now
			// so we have to check if it's still connected before
			// we go on to input processing
			if (!tcp.isConnected())
				continue;

			tcp.getline(line, 255);

			buff = string(line);
			Stripper stripper;
			buff = stripper.Strip(buff);
			
			if (buff.empty())
				continue;

			vector<string> cmdarg = ProcessRequest(buff);
			// process command, return results
			string cmd = cmdarg[0];

			vector<string> lines;

			if (cmd == "LIST") {
				mtx->EnterMutex();
				lines = retInterfaces();
				mtx->LeaveMutex();
			}
			else if (cmd == "DATA") {
				mtx->EnterMutex();
				lines = retData(cmdarg);
				mtx->LeaveMutex();
			}
			else if (cmd == "ATTACKS")
				lines = retAttacks(cmdarg);

			for (vector<string>::iterator it = lines.begin(); it != lines.end(); it++)
				tcp << *it << "\n";
			tcp << ".\n";
			tcp.close();
		}
}

vector<string> WebBroker::ProcessRequest(string buff){
	vector<string> cmdnargs;
	
	// If the data that was sent to us was of zero length (i.e. connection was closed before
	// we manage to read in any data), return
	if (buff.empty())
		return cmdnargs;

	// ...or else read the data in "mstr"
	char *mstr = (char *)buff.c_str();
	
	// Parse it with StringTokenizer
	StringTokenizer tok(mstr);
	StringTokenizer::iterator it = tok.begin();

	// Find the command
	cmdnargs.push_back(string(*it));
	++it;
	
	while (it != tok.end()){
		cmdnargs.push_back(string(*it));
		++it;
	}
	
	return cmdnargs;
}

vector<string> WebBroker::retInterfaces(){
	vector<string> lines;

	TimeSlot *ts = collector->getProcTS();
	if (!ts)
		return lines;
	map< uint16_t, map <uint16_t, IfPair *> > *inttree = ts->getIntTree();
	IfPair *curif;

	for (map<uint16_t, map< uint16_t, IfPair *> >::iterator it = (*inttree).begin(); it != (*inttree).end(); it++)
		for (map<uint16_t, IfPair *>::iterator dit = (*it).second.begin(); dit != (*it).second.end(); dit++){
			curif = (*dit).second;
			InetHostAddress h = curif->getOutIP();
			struct in_addr in = h.getAddress();
			lines.push_back(string(inet_ntoa(in)));
			break;
		}

	return lines;
}

vector<string> WebBroker::retData(vector<string> args){
	vector<string> lines;
	bool condition;
	char str[512];

	if (args.size() < 4)
		return lines;

	string cmd = args[1];

	char *mstr = (char *)args[2].c_str();
	InetHostAddress input = InetHostAddress((const char *)mstr);

	char *mstr2 = (char *)args[3].c_str();
	InetHostAddress output = InetHostAddress((const char *)mstr2);

	TimeSlot *ts = collector->getProcTS();
	if (!ts)
		return lines;
	map< uint16_t, map <uint16_t, IfPair *> > *inttree = ts->getIntTree();
	IfPair *curif;
	
	for (map<uint16_t, map< uint16_t, IfPair *> >::iterator it = (*inttree).begin(); it != (*inttree).end(); it++)
		for (map<uint16_t, IfPair *>::iterator dit = (*it).second.begin(); dit != (*it).second.end(); dit++){
			curif = (*dit).second;
			condition = false;
			if (args[2] != string("0") && args[3] != string("0"))
				condition = (input == curif->getFromIP() && output == curif->getOutIP());
			else if (args[2] != string("0"))
				condition = (input == curif->getFromIP());
			else if ( args[3] != string("0"))
				condition =  (output == curif->getOutIP());
			else
				condition = true;
			
			if (condition){
				memset((void *)str, '\0', 512);
				uint32_t pf = curif->getflows();
				uint32_t pp = curif->getpackets();

				if (pf && pp){
					InetHostAddress ttf = curif->getTopTalker(TALK_FLOWS);
					InetHostAddress ttp = curif->getTopTalker(TALK_PACKETS);
					InetHostAddress inif = curif->getFromIP();
					InetHostAddress outif = curif->getOutIP();

					struct in_addr f_addr, p_addr, in, out;

					f_addr = ttf.getAddress();
					p_addr = ttp.getAddress();
					in = inif.getAddress();
					out = outif.getAddress();

					uint32_t hflows = curif->getflows(ttf);
					uint32_t hpackets = curif->getpackets(ttp);
					
					sprintf(str, "%d %d", pf, pp);
					string sstr = string(str);
					sstr = sstr + string(" ") + string(inet_ntoa(in)) + \
							string(" ") + string(inet_ntoa(out)) + \
							string(" ") + string(inet_ntoa(f_addr)) + \
							string(" ") + string(inet_ntoa(p_addr));
					sprintf(str, " %d %d", hflows, hpackets);
					sstr = sstr + string(str);
					lines.push_back(sstr);
				}
			}
		}

	return lines;
}

vector<string> WebBroker::retAttacks(vector<string> args){
	vector<string> lines;
	
	return lines;
}
